// 
// Better ADL Version 2
// Copyright (C) 2016, NorthTrader.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
    /// </summary>
    [Description("Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.")]
    public class BADL : Indicator
    {
        #region Variables

		//Parameter defaults
		int iEMAPeriod = 50;
		int iSubBarPeriod = 1;
		bool bIncludeGaps = true;
		bool bDebug = false;

		//Other
		bool bFirstBarOnChart, bSameAsChart, bEMA, bSetBADL;
		bool bFirstEMA, bFirstSub, bFirstRealTimeSub, bMainBarEnd;
		int iSubCount, iReducer, iLastMainBar, iLastSubBar;
		double dSubOpen, dSubClose, dSubBAD, dSubVol, dBarBAD, dBarVol;
		double dMainOpen, dMainClose, dMainVol, dBADL, dLastBADL; 
		DateTime tSubFirst, tSubLast, tMain, tSubExpected;

		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			CalculateOnBarClose = false; 　//Will update in real time
			BarsRequired = 0;
			Add(new Plot(Color.DarkBlue, PlotStyle.Line, "BADL"));
			if (iSubBarPeriod > 0) Add(PeriodType.Minute, iSubBarPeriod);  //Add sub data series if necessary
			if (iEMAPeriod > 0)
			{
				//Add EMA plot if necessary
				bEMA = bFirstEMA = true;
				Add(new Plot(Color.Orange, "EMA"));
			}
			else bEMA = false;
			bFirstBarOnChart = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick) if CalculateOnBarClose = false. Otherwise called on bar close.
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (bFirstBarOnChart) SetVars();

			if (BarsInProgress == 0)
			{
				//MAIN data series (bars shown on chart)
				if (Historical || (iLastMainBar == -1))
				{
					//Every historical completed main bar, or last (incomplete) historical main bar 
					SetMain(0);
					if (!Historical)
					{
						//First non-historical update, so set vars for real time processing 
						bMainBarEnd = false;
						iLastMainBar = CurrentBar;
						if (bSameAsChart)
						{
							//Reset BADL so any partial bar value isn't included twice in real time
							if (bDebug) Print("---- This BADL is temporary until bar close if running in real time");
							dBADL = dLastBADL;
						}
					}
					if (bSameAsChart)
					{
						//Sub and Main are the same, so value is for Main bar
						if (Historical)
						{
							//Don't set BADL for first non-historical update
							dSubBAD = dBarValue(0);
							SetBADL(false, false);
						}
					}
					else
					{
						bSetBADL = true;  //Set later
						tSubExpected = tMain.AddMinutes(-iSubBarPeriod);
					}
				}
				else if (CurrentBar != iLastMainBar)
				{
					//Change of main bar in real time
					SetMain(1);
					bMainBarEnd = true;
					if (bSameAsChart)
					{
						dSubBAD = dBarValue(1);  //Sub and Main are the same, so value is for Main bar
						SetBADL(false, false);
					}
					iLastMainBar = CurrentBar;
				}
				//End MAIN data series
			}
			else if (BarsInProgress == 1 && !bSameAsChart)
			{
				//SUB data series (lower-timeframe bars used for calculation)
				if (Historical)
				{
					//Every historical completed sub bar
					if (bSetBADL && tSubLast < tSubExpected)
					{
						//BADL was not set for previous Main bar (because actual duration was less than bar period), so set it now instead
						SetBADL(true, true);
					}
					
					SetSub(0);
					if (bSetBADL)
					{
						//Last sub bar of main bar
						SetBADL(true, false);
					}
				}
				else if (iLastSubBar == -1)
				{
					//The last (incomplete) historical sub bar
					SetBADL(true, false);
					iLastSubBar = CurrentBar;
					bFirstRealTimeSub = true;
				}
				else if (CurrentBar != iLastSubBar)
				{
					//Change of sub bar in real time
					SetSub(1);
					SetBADL(true, false);
					iLastSubBar = CurrentBar;
					bMainBarEnd = false;
				}
				//End SUB data series
			}
        }

		#region Functions

		private double dBarValue(int aiBar)
		{
			//Return the BAD value of the bar
			//aiBar is 0 for current bar, 1 for previous bar
			double ldOpen = bIncludeGaps && (CurrentBar != 0) ? Close[aiBar + 1] : Open[aiBar];
			double ldHigh = Math.Max(ldOpen, High[aiBar]);
			double ldLow = Math.Min(ldOpen, Low[aiBar]);
			double ldSpread = Close[aiBar] - ldOpen;  //can be negative
			double ldSpreadTicks = Math.Abs(ldSpread) / TickSize;  //always positive
			double ldSpreadRatio = ldSpread / (ldHigh - ldLow);
			
			return ldHigh == ldLow ? 0 : (ldSpreadRatio * Volume[aiBar] * ldSpreadTicks);
		}

		private void SetBADL(bool abSub, bool abPremature)
		{
			//Set the BADL (running total)
			//Called on close of last Sub Bar of each each Main Bar for historical bars,
			//and on the close of each Sub Bar in real time
			double ldSubBAD = (iSubCount == 0 ? 0 : dSubBAD / iSubCount);
			double ldBADL = dBADL + ldSubBAD;
			int liBar = (Historical || !bMainBarEnd ? 0 : 1);
			dLastBADL = dBADL;

			BAD.Set(liBar, ldBADL);
			if (bMainBarEnd)
			{
				dBADL = ldBADL;
				if (bEMA)
				{
					if (bFirstEMA)
					{
						EMA.Set(BAD[0]);
						bFirstEMA = false;
					}
					else EMA.Set(liBar, BAD[liBar] * (2.0 / (1 + iEMAPeriod)) + (1 - (2.0 / (1 + iEMAPeriod))) * EMA[liBar + 1]);
				}
			}
			if (bDebug)
			{
				string lsBAD = ", BAD=" + sVal(ldSubBAD) + ", BADL=" + sVal(ldBADL) + (bMainBarEnd ? "" : " (temp)");
				Print("Set BADL from " + (abSub ? "Sub" : "Main") + " bar" + (abPremature ? " (for partial Main bar)" : "") + ", Historical=" + Historical);
				if (bMainBarEnd) Print("---- Main=" + sDateTime(tMain) + ", O-C=" + dMainOpen + "-" + dMainClose + ", V=" + dMainVol + (bSameAsChart ? lsBAD : ""));
				if (!bSameAsChart) Print("---- Subs=" + tSubFirst.ToString("dd-MMM HH:mm") +  " to " + tSubLast.ToString("dd-MMM HH:mm") +
					", O-C=" + dSubOpen + "-" + dSubClose +  ", V=" + dSubVol + ", Bars=" + iSubCount + lsBAD);
			}
			if (bMainBarEnd)
			{
				//Reset vars
				dSubBAD = dSubVol = 0;
				iSubCount = (bSameAsChart ? iReducer : 0);
				bFirstSub = true;
			}
			bSetBADL = false;
		}

		private void SetSub(int aiBar)
		{
			//Set the Sub Bar variables
			//aiBar is 0 for current bar, 1 for previous bar
			string lsMsg = "";
			if (aiBar == 1 && bFirstRealTimeSub && iSubCount > 0)
			{
				//First real time sub bar close, so remove partial bar values from totals
				dSubBAD -= dBarBAD;
				dSubVol -= dBarVol;
				iSubCount --;
				if (bDebug) lsMsg = " (first real time Sub bar close)";
				bFirstRealTimeSub = false;
			}
			dBarBAD = dBarValue(aiBar) / iSubBarPeriod;
			dBarVol = Volume[aiBar];
			dSubBAD += dBarBAD;
			dSubVol += dBarVol;
			iSubCount ++;
			tSubLast = Time[aiBar];
			dSubClose = Close[aiBar];
			if (bFirstSub)
			{
				//First sub bar of a new main bar
				tSubFirst = tSubLast;
				dSubOpen = Open[aiBar];
				bFirstSub = false;
			}
			if (bDebug) Print("SetSub " + tSubLast.ToString("HH:mm") + lsMsg + ", C=" + dSubClose + ", V=" + dBarVol + ", BAD=" + sVal(dBarBAD));
		}

		private void SetMain(int aiBar)
		{
			//Set the Main Bar variables
			//aiBar is 0 for current bar, 1 for previous bar
			tMain = Time[aiBar];
			dMainOpen = Open[aiBar];
			dMainClose = Close[aiBar];
			dMainVol = Volume[aiBar];
		}

		private string sDateTime(DateTime atD)
		{
			//Return DateTime as a standard formatted string
			return atD.ToString("dd-MMM-yyyy HH:mm");
		}

		private string sVal(double adV)
		{
			//Return decimal number as a string with one decimal place
			return adV.ToString("0.0");
		}

		private void SetVars()
		{
			//Called once on first bar of chart (for either data series)
			//Accesses properties that are unavailable in Initialize()
			bFirstBarOnChart = false;
			string lsMainBarType = BarsPeriod.Id.ToString().Substring(0, 1);
			if (iSubBarPeriod == 0 || (BarsPeriod.Id == PeriodType.Minute && BarsPeriod.Value == iSubBarPeriod))
			{
				//Only processing main bars, not sub bars
				bSameAsChart = true;
				//Set factor to reduce large numbers
				if (BarsPeriod.Id == PeriodType.Minute) iReducer = BarsPeriod.Value;
				else if (BarsPeriod.Id == PeriodType.Day) iReducer = 1000;
				else iReducer = 5000;
			}
			else bSameAsChart = false;
			dSubBAD = dSubVol = dMainVol = dBADL = 0;
			iSubCount = (bSameAsChart ? iReducer : 0);
			iLastMainBar = iLastSubBar = -1;
			bSetBADL = false;
			bFirstSub = bMainBarEnd = true;
			
			//Write header to Output window
			if (bDebug)
			{
				string lsSep = new String('=', 80);
				ClearOutputWindow();
				Print(lsSep);
				Print("BADL V2: " + Instrument.FullName + ", Period = " + BarsPeriod.Id.ToString().Substring(0, 1) + BarsPeriod.Value +
					", Pip Value = " + TickSize.ToString() + ", Run time = " + sDateTime(DateTime.Now) + " (local)");
				Print("---- Sub-bar calculation period = " + (bSameAsChart ? " same as chart bars" : "M" + iSubBarPeriod) +
					", Include gap spreads = " + bIncludeGaps);
				Print("---- Calculate on bar close = " + CalculateOnBarClose + ", Debug mode = " + bDebug);
				if (bSameAsChart) Print("---- Reducer = " + iReducer);
				Print(lsSep);
			}
		}

		#endregion	
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries BAD
        {
            get { return Values[0]; }
        }
        [Browsable(false)]
        [XmlIgnore()]
        public DataSeries EMA
        {
            get { return Values[1]; }
        }

		//Parameters 
		[Description("Period of EMA (in bars). 0 = no EMA.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("\t\tPeriod of EMA")]
		public int EMAPeriod
		{
            get { return iEMAPeriod; }
            set { iEMAPeriod = Math.Max(0, value); }
        }

		[Description("Period of each sub-bar (in minutes) on which the calculation is performed. 0 = same as chart bars.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("\t\tPeriod of Sub-Bar")]
		public int SubBarPeriod
		{
            get { return iSubBarPeriod; }
            set { iSubBarPeriod = Math.Max(0, value); }
        }

		[Description("True = use previous bar's Close for current bar's Open (include gap spreads). False = always use current bar's Open (ignore gap spreads).")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("\tInclude gap spreads")]
		public bool IncludeGaps
		{
            get { return bIncludeGaps; }
            set { bIncludeGaps = value; }
        }

		[Description("True = write run-time information to Output window. Set to False unless testing.")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayName("Debug mode")]
		public bool DebugMode
		{
            get { return bDebug; }
            set { bDebug = value; }
        }

		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private BADL[] cacheBADL = null;

        private static BADL checkBADL = new BADL();

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public BADL BADL(bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            return BADL(Input, debugMode, eMAPeriod, includeGaps, subBarPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public BADL BADL(Data.IDataSeries input, bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            if (cacheBADL != null)
                for (int idx = 0; idx < cacheBADL.Length; idx++)
                    if (cacheBADL[idx].DebugMode == debugMode && cacheBADL[idx].EMAPeriod == eMAPeriod && cacheBADL[idx].IncludeGaps == includeGaps && cacheBADL[idx].SubBarPeriod == subBarPeriod && cacheBADL[idx].EqualsInput(input))
                        return cacheBADL[idx];

            lock (checkBADL)
            {
                checkBADL.DebugMode = debugMode;
                debugMode = checkBADL.DebugMode;
                checkBADL.EMAPeriod = eMAPeriod;
                eMAPeriod = checkBADL.EMAPeriod;
                checkBADL.IncludeGaps = includeGaps;
                includeGaps = checkBADL.IncludeGaps;
                checkBADL.SubBarPeriod = subBarPeriod;
                subBarPeriod = checkBADL.SubBarPeriod;

                if (cacheBADL != null)
                    for (int idx = 0; idx < cacheBADL.Length; idx++)
                        if (cacheBADL[idx].DebugMode == debugMode && cacheBADL[idx].EMAPeriod == eMAPeriod && cacheBADL[idx].IncludeGaps == includeGaps && cacheBADL[idx].SubBarPeriod == subBarPeriod && cacheBADL[idx].EqualsInput(input))
                            return cacheBADL[idx];

                BADL indicator = new BADL();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.DebugMode = debugMode;
                indicator.EMAPeriod = eMAPeriod;
                indicator.IncludeGaps = includeGaps;
                indicator.SubBarPeriod = subBarPeriod;
                Indicators.Add(indicator);
                indicator.SetUp();

                BADL[] tmp = new BADL[cacheBADL == null ? 1 : cacheBADL.Length + 1];
                if (cacheBADL != null)
                    cacheBADL.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheBADL = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BADL BADL(bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            return _indicator.BADL(Input, debugMode, eMAPeriod, includeGaps, subBarPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.BADL BADL(Data.IDataSeries input, bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            return _indicator.BADL(input, debugMode, eMAPeriod, includeGaps, subBarPeriod);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.BADL BADL(bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            return _indicator.BADL(Input, debugMode, eMAPeriod, includeGaps, subBarPeriod);
        }

        /// <summary>
        /// Better ADL: an adaption of Chaikin's Accumulation/Distribution Line, which attempts to quantify the amount of volume flowing into or out of an instrument.
        /// </summary>
        /// <returns></returns>
        public Indicator.BADL BADL(Data.IDataSeries input, bool debugMode, int eMAPeriod, bool includeGaps, int subBarPeriod)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.BADL(input, debugMode, eMAPeriod, includeGaps, subBarPeriod);
        }
    }
}
#endregion
